Module jack::logging
[−]
[src]
Control error and info logging from JACK.
Functions
| get_error_callback |
Get the error callback that was set using |
| get_info_callback |
Resets the JACK info callback to use stdio.
Get the info callback that was set using |
| reset_error_callback |
Restores the JACK info callback to the JACK default, which is to write to stderr. |
| reset_info_callback |
Restores the JACK info callback to the JACK default, which is to write to stdout. |
| set_error_callback |
Set the global JACK error callback. It is recommended to use the log crate. |
| set_info_callback |
Set the global JACK info callback. It is recommended to use the log crate. |